/**
 * Author: Valerio Schiavoni <valerio.schiavoni@gmail.com>
 */

package org.objectweb.fractal.bf.hdl.graph;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.logging.Logger;

import jdsl.core.util.Converter;
import jdsl.graph.algo.InvalidQueryException;
import jdsl.graph.api.Edge;
import jdsl.graph.api.EdgeIterator;
import jdsl.graph.api.Graph;
import jdsl.graph.api.Vertex;
import jdsl.graph.api.VertexIterator;

import org.objectweb.fractal.bf.hdl.reader.ShortestPathExecutor;

/**
 * Given an input graph, for any given {@link PairOfVertices} this resolver returns the shortest path as a set of iterable edges.
 */
public class ShortestPathResolver
{

  Logger                            log = Logger
                                            .getLogger(ShortestPathResolver.class
                                                .getName());

  /**
   * Easy access to shortest path for every pair of vertices.
   */
  Map<PairOfVertices, Path> shortestPathsForPairs;

  Graph                             graph;

  ShortestPathExecutor              shortestPathExecutor;

  public ShortestPathResolver(Graph g)
  {
    this.graph = g;
    this.shortestPathExecutor = new ShortestPathExecutor();
    init();
  }

  /**
   * Fill the shortestPathForPairs map
   */
  private void init()
  {
    shortestPathsForPairs = new HashMap<PairOfVertices, Path>();

    final VertexIterator ve = graph.vertices();
    while (ve.hasNext())
    {
      Vertex aVertex = ve.nextVertex();

      VertexIterator anotherVertex = graph.vertices();
      while (anotherVertex.hasNext())
      {
        Vertex another = anotherVertex.nextVertex();

        if (aVertex.equals(another))
        {
          // skipping paths from vertex v to vertex v
          continue; // go to the next one
        }
        else
        {

          log.info("Computing shortest path between: " + aVertex + " and "
              + another);
          computeShortestPathBetween(aVertex, another);

          shortestPathExecutor.cleanup(); // better to cleanup between two
                                          // subsequent executions
          
          log.info("Computing shortest path between: " + another + " and "
              + aVertex);
          computeShortestPathBetween(another, aVertex); //swap vertices...
        }
      }
    }
  }

  /**
   * @param v1 from
   * @param v2 to
   */
  private void computeShortestPathBetween(Vertex v1, Vertex v2)
  {
    shortestPathExecutor.execute(this.graph, v1, v2);
    PairOfVertices pov = new PairOfVertices(v1, v2);
    try
    {
      final EdgeIterator reportPath = shortestPathExecutor.reportPath();
      
      final LinkedList<Edge> pathAsList = converteEdgeIteratorToPath(reportPath);
      
      Path path = new Path(graph,pathAsList);
      
      this.shortestPathsForPairs.put(pov, path);
    }
    catch (InvalidQueryException iqe)
    {
      this.shortestPathsForPairs.put(pov, null);
    }
  }


  public Path pathFor(PairOfVertices pov)
  {
    
    return this.shortestPathsForPairs.get(pov);
  }

  /**
   * @param iterator
   * @return 
   */
  private LinkedList<Edge> converteEdgeIteratorToPath(EdgeIterator iterator)
  {
    LinkedList<Edge> edgesInPath = new LinkedList<Edge>();
    
    Converter.iteratorToList(iterator, edgesInPath);
    
    return edgesInPath;
  }


}
